import { Meta } from "@storybook/addon-docs";

<Meta title="Introduction/Developer Guides/Data Test Attributes" />

# Developer Guides

## Data Test Attributes

It is important to have reliable selectors for testing purposes that can be used across platforms. Ideally these
selectors should be unique and can be included directly in components or static templates.

<strong>Naming Convention Attribute Name</strong>

`data-testid`

<strong>Naming Convention Attribute Value</strong>

`location-elementName-elementType`

<strong>Location</strong>

This could relate to the template or page the data test attribute is used for

<strong>Element Name</strong>

This could relate to an element name, component name or a specific action name the data test attribute is used for

<strong>Element Type</strong>

This could relate to an element type, component type or a specific action type the data test attribute is used for

<strong>Example 1</strong>

Test of button click "Request E-Catalog" on MyMachine Detail Page

Location is `myMachineDetail` followed by `-` followed by ElementName which in that case is `requestECatalog` followed by `-` followed by ElementType which in that case is `button`.

`myMachineDetail-requestECatalog-button`

```
<ux-button data-testid="myMachineDetail-requestECatalog-button">
    Request E-Catalog
</ux-button>
```

<strong>Example 2</strong>

Test of specific button click on MyMachine Overview Table List View that leads to MyMachine Detail Page:

Location is `myMachineOverview` followed by `-` followed by ElementName which in that case is `machineDetail` followed by `-` followed by ElementType which in that case is `button`.
If there are multiple buttons in a list or table view, the data test attribute value could be extended with a unique identifier, i.e. Machine Number.

`myMachineOverview-machineDetail-button{MachineNumber}`

```
<ux-table>
    <table>
        <thead>
            <tr>
                <th>Machine Number</th>
                <th>Year</th>
                <th>Sub Equipment</th>
                <th>Location</th>
                <th>Details</th>
            </tr>
        </thead>
        <tbody>
            <tr>
                <td>TWDEDM104</td>
                <td>2020</td>
                <td>5 Equipments</td>
                <td>Ditzingen</td>
                <td>
                    <ux-button variant="default" size="icon" iconplacement="start" icon="chevron-right" data-testid="myMachineOverview-machineDetail-buttonTWDEDM104"></ux-button>
                </td>
            </tr>
            <tr>

                <td>TWDEDM105</td>
                <td>2020</td>
                <td>5 Equipments</td>
                <td>Ditzingen</td>
                <td>
                    <ux-button variant="default" size="icon" iconplacement="start" icon="chevron-right" data-testid="myMachineOverview-machineDetail-buttonTWDEDM105"></ux-button>
                </td>
            </tr>
        </tbody>
    </table>
</ux-table>
```

<strong>Additional Info</strong>

Allowed Characters for HTML attributes:

<a
    href="https://html.spec.whatwg.org/multipage/syntax.html#attributes-2"
    target="_blank"
>
    https://html.spec.whatwg.org/multipage/syntax.html#attributes-2
</a>

Playwright usage example:

<a
    href="https://playwright.dev/docs/locators#locate-by-test-id"
    target="_blank"
>
    https://playwright.dev/docs/locators#locate-by-test-id
</a>
